const webpack = require('webpack')
const merge = require('webpack-merge')
const webpackBase = require('./webpack.base.config')
const { projectPath } = require('../utils')
const WorkboxPlugin = require('workbox-webpack-plugin')
const multi = require('multi-loader')
const appConfig = require('../utils/get-app-config')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const cleanWebpackPlugin = require('clean-webpack-plugin')
const Visualizer = require('webpack-visualizer-plugin')

const publicPath = appConfig.deploy && appConfig.deploy.project || 'public'

const config = merge(webpackBase, {
  mode: 'production',
  devtool: '#cheap-source-map',
  output: {
    filename: '[name]_[hash:6].js',
    path: `${projectPath}/${publicPath}`,
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: 'css/[name].css',
      chunkFilename: 'css/app.[contenthash:12].css',
    }),
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"',
      },
    }),
    new WorkboxPlugin.GenerateSW({
      // 这些选项帮助 ServiceWorkers 快速启用
      // 不允许遗留任何“旧的” ServiceWorkers
      clientsClaim: true,
      skipWaiting: true
    }),
    new Visualizer({
      filename: './statistics/index.html',
    }),
  ],
  optimization: {
    runtimeChunk: {
      name: 'manifest',
    },
    minimize: true,
    splitChunks: {
      cacheGroups: {
        styles: {
          name: 'styles',
          test: /.css$/,
          chunks: 'all',
          enforce: true,
        },
      },
    },
  },
})

// 增加清除dir命令
if (appConfig.deploy && appConfig.deploy.project && appConfig.deploy.isClean) {
  config.plugins.push(
    new cleanWebpackPlugin(appConfig.deploy.project, {
      root: projectPath,
    })
  )
}

// 如果为服务端渲染
if (appConfig.deploy && appConfig.deploy.isSSR) {
  config.output.publicPath = `/${publicPath}/`
} else {
  // 前端渲染
  config.output.publicPath = '/'
}

module.exports = config
